/*
 * Written by Sean Kelly
 * Placed into Public Domain
 */

module std.c.posix.signal;

public import std.c.signal;
public import std.c.posix.sys.types; // for pid_t

extern (C):

private alias void function(int) sigfn_t;
private alias void function(int, siginfo_t*, void*) sigactfn_t;

const sigfn_t SIG_HOLD = cast(sigfn_t) 2;

/+
union sigval
{
    int     sival_int;
    void*   sival_ptr;
}

struct sigevent
{
    int                     sigev_notify,
                            sigev_signo;
    sigval                  sigev_value;
    void function(sigval)   sigev_notify_function;
    pthread_attr_t*         sigev_notify_attributes;
}
+/

version( linux )
{
    struct sigset_t
    {
        uint __val[1024 / (8 * uint.sizeof)];
    }

    struct sigaction_t
    {
        __sighandler_t  sa_handler;
        sigset_t        sa_mask;
        int             sa_flags;
        void (*sa_restorer)();
    }
}

/+
const int   SIGEV_NONE   = ?;
const int   SIGEV_SIGNAL = ?;
const int   SIGEV_THREAD = ?;

const int   SIGRTMIN     = ?;
const int   SIGRTMAX     = ?;
+/

//const int SIGABRT = 6;  // Abnormal termination
const int SIGALRM = 14; // Time-out alarm
const int SIGBUS  = 10; // Access to an undefined portion of a memory object
const int SIGCHLD = 18; // Change in status of child
const int SIGCONT = 25; // Continue stopped process
//const int SIGFPE  = 8;  // Floating-point error
const int SIGHUP  = 1;  // Hangup
//const int SIGILL  = 4;  // Illegal hardware instruction
//const int SIGINT  = 2;  // Terminal interrupt character
const int SIGKILL = 9;  // Termination
const int SIGPIPE = 13; // Write to pipe with no readers
const int SIGQUIT = 3;  // Terminal Quit character
//const int SIGSEGV = 11; // Invalid memory reference
const int SIGSTOP = 23; // Stop process
//const int SIGTERM = 15; // Termination
const int SIGTSTP = 24; // Terminal Stop character
const int SIGTTIN = 26; // Background read from control tty
const int SIGTTOU = 27; // Background write to control tty
const int SIGUSR1 = 16; // User defined signal
const int SIGUSR2 = 17; // User defined signal
const int SIGURG  = 21; // High bandwidth data is available at a socket

int kill(pid_t, int);
int sigaction(int, sigaction_t*, sigaction_t*);
int sigaddset(sigset_t*, int);
int sigdelset(sigset_t*, int);
int sigemptyset(sigset_t*);
int sigfillset(sigset_t*);
int sigismember(sigset_t*, int);
int sigpending(sigset_t*);
int sigprocmask(int, sigset_t*, sigset_t*);
int sigsuspend(sigset_t*);
int sigwait(sigset_t*, int*);

//
// XOpen (XSI)
//
/*
private import std.c.stddef;
public import std.c.posix.ucontext;

struct stack_t
{
    void*   ss_sp;
    size_t  ss_size;
    int     ss_flags;
}

struct sigstack
{
    int     ss_onstack;
    void*   ss_sp;
}

const int SIGPOLL   = ??; // Pollable event
const int SIGPROF   = ??; // Profiling timer expired
const int SIGSYS    = ??; // Bad system call
const int SIGTRAP   = ??; // Trace/breakpoint trap
const int SIGVTALRM = ??; // Virtual timer expired
const int SIGXCPU   = ??; // CPU time limit exceeded
const int SIGXFSZ   = ??; // File size limit exceeded

SA_ONSTACK
SA_RESETHAND
SA_RESTART
SA_SIGINFO
SA_NOCLDWAIT
SA_NODEFER
SS_ONSTACK
SS_DISABLE
MINSIGSTKSZ
SIGSTKSZ

int     si_errno;
pid_t   si_pid;
uid_t   si_uid;
void*   si_addr;
int     si_status;
int     si_band;

ILL_ILLOPC
ILL_ILLOPN
ILL_ILLADR
ILL_ILLTRP
ILL_PRVOPC
ILL_PRVREG
ILL_COPROC
ILL_BADSTK
FPE_INTDIV
FPE_INTOVF
FPE_FLTDIV
FPE_FLTOVF
FPE_FLTUND
FPE_FLTRES
FPE_FLTINV
FPE_FLTSUB
SEGV_MAPERR
SEGV_ACCERR
BUS_ADRALN
BUS_ADRERR
BUS_OBJERR
TRAP_BRKPT
TRAP_TRACE
CLD_EXITED
CLD_KILLED
CLD_DUMPED
CLD_TRAPPED
CLD_STOPPED
CLD_CONTINUED
POLL_IN
POLL_OUT
POLL_MSG
POLL_ERR
POLL_PRI
POLL_HUP

void*   si_addr;
pid_t   si_pid;
int     si_status;
uid_t   si_uid;
int     si_band;

sigfn_t bsd_signal(int sig, sigfn_t func);

int killpg(pid_t, int);
int sigaltstack(stack_t*, stack_t*);
int sighold(int);
int sigignore(int);
int siginterrupt(int, int);
int sigpause(int);
int sigrelse(int);
sigfn_t sigset(int sig, sigfn_t func);
*/

//
// Realtime Signals (RTS)
//
/*
sigval  si_value;

int sigqueue(pid_t, int, sigval);
int sigtimedwait(sigset_t*, siginfo_t*, timespec*);
int sigwaitinfo(sigset_t*, siginfo_t*);
*/

version( linux )
{
    int sigtimedwait(sigset_t*, siginfo_t*, timespec*);
}


//
// Threads (THR)
//
/*
int pthread_kill(pthread_t, int);
int pthread_sigmask(int, sigset_t*, sigset_t*);
*/