/*
 * Written by Sean Kelly
 * Placed into Public Domain
 */

module std.c.posix.sys.types;

private import std.c.stdint;
public import std.c.stddef; // needed for size_t
public import std.c.time;   // needed for time_t

extern (C):

//
// Required
//
/*
blkcnt_t
blksize_t
dev_t
gid_t
ino_t
mode_t
nlink_t
off_t
pid_t
size_t
ssize_t
time_t
uid_t
*/
version( linux )
{
    //blkcnt_t
    //blksize_t
    alias int       dev_t;
    alias int       gid_t;
    alias uint      ino_t;
    alias uint      mode_t;
    alias uint      nlink_t;
    alias int       off_t;
    alias int       pid_t;
    //size_t (defined in std.c.stddef)
    alias size_t    ssize_t;
    //time_t (defined in std.c.time)
    alias uint      uid_t;
}


//
// XOpen (XSI)
//
/*
clock_t
fsblkcnt_t
fsfilcnt_t
id_t
key_t
suseconds_t
useconds_t
*/


//
// Thread (THR)
//
/*
pthread_attr_t
pthread_cond_t
pthread_condattr_t
pthread_key_t
pthread_mutex_t
pthread_mutexattr_t
pthread_once_t
pthread_rwlock_t
pthread_rwlockattr_t
pthread_t
*/
version( linux )
{
    private struct _pthread_fastlock
    {
        int __status;
        int __spinlock;
    }

    struct pthread_attr_t
    {
        int             __detachstate;
        int             __schedpolicy;
        __sched_param   __schedparam; // defined in std.c.posix.sched
        int             __inheritsched;
        int             __scope;
        size_t          __guardsize;
        int             __stackaddr_set;
        void*           __stackaddr;
        size_t          __stacksize;
    }
/+
    struct pthread_cond_t
    {
        _pthread_fastlock __c_lock;
        _pthread_descr __c_waiting; // don't have definition of this yet
        char __padding[48 - _pthread_fastlock.sizeof
		                  - _pthread_descr.sizeof
		                  - __pthread_cond_align_t.sizeof];
        __pthread_cond_align_t __align;
    }
+/
    struct pthread_condattr_t
    {
        int __dummy;
    }

    alias uint pthread_key_t;

    struct pthread_mutex_t
    {
        int __m_reserved;
        int __m_count;
        _pthread_descr __m_owner;
        int __m_kind;
        _pthread_fastlock __m_lock;
    }

    struct pthread_mutexattr_t
    {
        int __mutexkind;
    }

    alias int pthread_once_t;

    struct pthread_rwlock_t
    {
        _pthread_fastlock __rw_lock;
        int __rw_readers;
        _pthread_descr __rw_writer;
        _pthread_descr __rw_read_waiting;
        _pthread_descr __rw_write_waiting;
        int __rw_kind;
        int __rw_pshared;
    }

    struct pthread_rwlockattr_t
    {
        int __lockkind;
        int __pshared;
    }

    alias uint pthread_t;
}


//
// Barrier (BAR)
//
/*
pthread_barrier_t
pthread_barrierattr_t
*/


//
// Spin (SPN)
//
/*
pthread_spinlock_t
*/


//
// Timer (TMR)
//
/*
clockid_t
timer_t
*/


//
// Trace (TRC)
//
/*
trace_attr_t
trace_event_id_t
trace_event_set_t
trace_id_t
*/