module std.memory;

private
{
    extern (C) void gc_init();
    extern (C) void gc_term();

    alias void function( void *p, void *dummy ) gc_finalizer;
    extern (C) void gc_setFinalizer( void *p, gc_finalizer fn );

    extern (C) void gc_enable();
    extern (C) void gc_disable();
    extern (C) void gc_collect();

    extern (C) void* gc_malloc( size_t sz );
    extern (C) void* gc_calloc( size_t nm, size_t sz );
    extern (C) void* gc_realloc( void* p, size_t sz );
    extern (C) void gc_free( void* p );

    extern (C) size_t gc_capacityOf( void* p );

    extern (C) void gc_addRoot( void* p );
    extern (C) void gc_addRange( void* pbeg, void* pend );

    extern (C) void gc_removeRoot( void* p );
    extern (C) void gc_removeRange( void* pbeg, void* pend );

    extern (C) void gc_pin( void* p );
    extern (C) void gc_unpin( void* p );
}

struct GC
{
    void enable()  { gc_enable();  }
    void disable() { gc_disable(); }
    void collect() { gc_collect(); }

    void* malloc( size_t sz )            { return gc_malloc( sz );     }
    void* calloc( size_t nm, size_t sz ) { return gc_calloc( nm, sz ); }
    void* realloc( void* p, size_t sz )  { return gc_realloc( p, sz ); }
    void free( void* p )                 { gc_free( p );               }

    size_t capacityOf( void* p ) { return gc_capacityOf( p ); }

    void add( void* p )                { gc_addRoot( p );           }
    void add( void* pbeg, void* pend ) { gc_addRange( pbeg, pend ); }

    void remove( void* p )                { gc_removeRoot( p );           }
    void remove( void* pbeg, void* pend ) { gc_removeRange( pbeg, pend ); }

    void pin( void* p )   { gc_pin( p );   }
    void unpin( void* p ) { gc_unpin( p ); }
}

GC gc;