/*
 * Placed into the Public Domain.
 * written by Walter Bright
 * www.digitalmars.com
 */

private
{
    import std.c.stdlib;
    import std.c.string;
    import std.c.stdio;
}

extern (C) void _STI_monitor_staticctor();
extern (C) void _STD_monitor_staticdtor();
extern (C) void _STI_critical_init();
extern (C) void _STD_critical_term();
extern (C) void gc_init();
extern (C) void gc_term();
extern (C) void _minit();
extern (C) void _moduleCtor();
extern (C) void _moduleDtor();
extern (C) void _moduleUnitTests();

/***********************************
 * These functions must be defined for any D program linked
 * against this library.
 */
extern (C) void onAssert( char[] file, uint line );
// the following functions are for internal use, and exist to avoid
// any object definition dependencies between the runtime and std lib
extern (C) void onArrayBoundsError( char[] file, uint line );
extern (C) void onSwitchError( char[] file, uint line );
// this function is called by [internal.utf] and exists to provide
// a unified method for UTF error handling
extern (C) void onInvalidUtfError( char[] msg, size_t idx );

/***********************************
 * These are internal callbacks for various language errors.
 */
extern (C) void _d_assert( char[] file, uint line )
{
    onAssert( file, line );
}

extern (C) void _d_array_bounds( char[] file, uint line )
{
    onArrayBoundsError( file, line );
}

extern (C) void _d_switch_error( char[] file, uint line )
{
    onSwitchError( file, line );
}

/***********************************
 * The D main() function supplied by the user's program
 */
int main(char[][] args);

/***********************************
 * Substitutes for the C main() function.
 * It's purpose is to wrap the call to the D main()
 * function and catch any unhandled exceptions.
 */

extern (C) int main(int argc, char **argv)
{
    char[] *am;
    char[][] args;
    int i;
    int result;
    int myesp;
    int myebx;

    version (linux)
    {
	    _STI_monitor_staticctor();
	    _STI_critical_init();
	    gc_init();
	    am = cast(char[] *) malloc(argc * (char[]).sizeof);
	    // BUG: alloca() conflicts with try-catch-finally stack unwinding
	    //am = (char[] *) alloca(argc * (char[]).sizeof);
    }
    version (Win32)
    {
	    gc_init();
	    _minit();
	    am = cast(char[] *) alloca(argc * (char[]).sizeof);
    }

    try
    {
	    _moduleCtor();
	    _moduleUnitTests();
    	for (i = 0; i < argc; i++)
    	{
    	    int len = strlen(argv[i]);
    	    am[i] = argv[i][0 .. len];
    	}

    	args = am[0 .. argc];

    	result = main(args);
	    _moduleDtor();
	    gc_term();
    }
    catch (Exception e)
    {
        if (e.file)
	        fprintf(stderr, "%.*s(%u): %.*s\n", e.file, e.line, e.msg);
	    else
	        fprintf(stderr, "%.*s\n", e.toString());
	    exit(EXIT_FAILURE);
    }
    catch (Object o)
    {
	    fprintf(stderr, "%.*s\n", o.toString() );
	    exit(EXIT_FAILURE);
    }

    version (linux)
    {
	    free(am);
	    _STD_critical_term();
	    _STD_monitor_staticdtor();
    }
    return result;
}