/*
 *  Copyright (C) 2004 by Digital Mars, www.digitalmars.com
 *  Written by Walter Bright
 *
 *  This software is provided 'as-is', without any express or implied
 *  warranty. In no event will the authors be held liable for any damages
 *  arising from the use of this software.
 *
 *  Permission is granted to anyone to use this software for any purpose,
 *  including commercial applications, and to alter it and redistribute it
 *  freely, in both source and binary form, subject to the following
 *  restrictions:
 *
 *  o  The origin of this software must not be misrepresented; you must not
 *     claim that you wrote the original software. If you use this software
 *     in a product, an acknowledgment in the product documentation would be
 *     appreciated but is not required.
 *  o  Altered source versions must be plainly marked as such, and must not
 *     be misrepresented as being the original software.
 *  o  This notice may not be removed or altered from any source
 *     distribution.
 */

private import std.c.string;

class TypeInfo_StaticArray : TypeInfo
{
    char[] toString()
    {
    // BUG: need to be able to convert int to string
	//return next.toString() ~ "[" ~ std.string.toString(len) ~ "]";
	return next.toString() ~ "[]";
    }

    uint getHash(void *p)
    {	size_t sz = next.tsize();
	uint hash = 0;
	for (size_t i = 0; i < len; i++)
	    hash += next.getHash(p + i * sz);
        return hash;
    }

    int equals(void *p1, void *p2)
    {
	size_t sz = next.tsize();

        for (size_t u = 0; u < len; u++)
        {
	    if (!next.equals(p1 + u * sz, p2 + u * sz))
		return 0;
        }
        return 1;
    }

    int compare(void *p1, void *p2)
    {
	size_t sz = next.tsize();

        for (size_t u = 0; u < len; u++)
        {
            int result = next.compare(p1 + u * sz, p2 + u * sz);
            if (result)
                return result;
        }
        return 0;
    }

    size_t tsize()
    {
	return len * next.tsize();
    }

    void swap(void *p1, void *p2)
    {	ubyte* tmp;
	size_t sz = next.tsize();
	ubyte[16] buffer;
	ubyte* pbuffer;

	if (sz < buffer.sizeof)
	    tmp = buffer;
	else
	    tmp = pbuffer = new ubyte[sz];

	for (size_t u = 0; u < len; u += sz)
	{   size_t o = u * sz;
	    memcpy(tmp, p1 + o, sz);
	    memcpy(p1 + o, p2 + o, sz);
	    memcpy(p2 + o, tmp, sz);
	}
	if (pbuffer)
	    delete pbuffer;
    }

    TypeInfo next;
    size_t len;
}