# Makefile to build D runtime library dmdrt.lib for Win32
# Designed to work with \dm\bin\make.exe
# Targets:
#	make
#		Same as make dmdrt
#	make dmdrt.lib
#		Build dmdrt.lib
#	make clean
#		Delete unneeded files created by build process
#	make unittest
#		Build dmdrt.lib, build and run unit tests
# Notes:
#	This relies on LIB.EXE 8.00 or later, and MAKE.EXE 5.01 or later.

CP=copy /y

CFLAGS=-mn -6 -r -Igc
#CFLAGS=-g -mn -6 -r -Igc

DFLAGS=-release -O -inline
#DFLAGS=-g -release

TFLAGS=-O -inline
#TFLAGS=-g

CC=\bin\dm\bin\dmc
DC=\bin\dmd\bin\dmd

.c.obj:
	$(CC) -c $(CFLAGS) $*

.cpp.obj:
	$(CC) -c $(CFLAGS) $*

.d.obj:
	$(DC) -c $(DFLAGS) $*

#.asm.obj:
#	$(CC) -c $*

targets : dmdrt

dmdrt : dmdrt.lib

OBJ= aaA.obj aApply.obj adi.obj arraycast.obj arraycat.obj cast.obj \
    complex.obj critical.obj deh.obj dmain2.obj invariant.obj memory.obj \
    memset.obj monitor.obj obj.obj object.obj qsort.obj switch.obj \
    moduleinit.obj utf.obj
# trace.obj

OBJ_TI= \
    ti_void.obj \
    \
    ti_char.obj ti_wchar.obj ti_dchar.obj \
    ti_bit.obj \
    ti_byte.obj ti_ubyte.obj \
    ti_short.obj ti_ushort.obj \
    ti_int.obj ti_uint.obj \
	ti_long.obj ti_ulong.obj \
	ti_float.obj ti_cfloat.obj ti_ifloat.obj \
	ti_double.obj ti_cdouble.obj ti_idouble.obj \
	ti_real.obj ti_creal.obj ti_ireal.obj \
	\
	ti_Array.obj ti_AssociativeArray.obj ti_StaticArray.obj \
	\
	ti_AC.obj \
    ti_Aa.obj ti_Awchar.obj ti_Adchar.obj \
    ti_Abit.obj \
    ti_Ag.obj ti_Aubyte.obj \
    ti_Ashort.obj ti_Aushort.obj \
    ti_Aint.obj ti_Auint.obj \
	ti_Along.obj ti_Aulong.obj \
	ti_Afloat.obj ti_Acfloat.obj \
	ti_Adouble.obj ti_Acdouble.obj \
	ti_Areal.obj ti_Acreal.obj \
	\
	ti_Pointer.obj ti_Delegate.obj ti_Function.obj \
	ti_Enum.obj ti_Class.obj ti_Struct.obj ti_Typedef.obj

dmdrt.lib : $(OBJ) $(OBJ_TI) minit.obj
    del dmdrt.lib
	lib -c -n dmdrt.lib $(OBJ) $(OBJ_TI) minit.obj

######################################################

### global

error.obj : error.d
    $(DC) -c $(DFLAGS) error.d

hook.obj : hook.d
	$(DC) -c $(DFLAGS) hook.d

aaA.obj : aaA.d
	$(DC) -c $(DFLAGS) aaA.d

aApply.obj : aApply.d
	$(DC) -c $(DFLAGS) aApply.d

adi.obj : adi.d
	$(DC) -c $(DFLAGS) adi.d

arraycast.obj : arraycast.d
	$(DC) -c $(DFLAGS) arraycast.d

arraycat.obj : arraycat.d
	$(DC) -c $(DFLAGS) arraycat.d

cast.obj : cast.d
	$(DC) -c $(DFLAGS) cast.d

complex.obj : complex.c
	$(CC) -c $(CFLAGS) complex.c

critical.obj : critical.c
	$(CC) -c $(CFLAGS) critical.c

deh.obj : mars.h deh.c
	$(CC) -c $(CFLAGS) deh.c

dmain2.obj : dmain2.d
	$(DC) -c $(DFLAGS) dmain2.d

invariant.obj : invariant.d
	$(DC) -c $(DFLAGS) invariant.d

memory.obj : memory.d
	$(DC) -c $(DFLAGS) memory.d

memset.obj : memset.d
	$(DC) -c $(DFLAGS) memset.d

#minit.obj : minit.asm
#   $(CC) -c minit.asm

monitor.obj : mars.h monitor.c
	$(CC) -c $(CFLAGS) monitor.c

obj.obj : obj.d
	$(DC) -c $(DFLAGS) obj.d

object.obj : object.d
	$(DC) -c $(DFLAGS) object.d

qsort.obj : qsort.d
	$(DC) -c $(DFLAGS) qsort.d

switch.obj : switch.d
	$(DC) -c $(DFLAGS) switch.d

moduleinit.obj : moduleinit.d
	$(DC) -c $(DFLAGS) moduleinit.d

### tacked on

utf.obj : utf.d
	$(DC) -c $(DFLAGS) utf.d

### typeinfo

ti_void.obj : typeinfo\ti_void.d
	$(DC) -c $(DFLAGS) typeinfo\ti_void.d

ti_char.obj : typeinfo\ti_char.d
	$(DC) -c $(DFLAGS) typeinfo\ti_char.d

ti_wchar.obj : typeinfo\ti_wchar.d
	$(DC) -c $(DFLAGS) typeinfo\ti_wchar.d

ti_dchar.obj : typeinfo\ti_dchar.d
	$(DC) -c $(DFLAGS) typeinfo\ti_dchar.d

ti_bit.obj : typeinfo\ti_bit.d
	$(DC) -c $(DFLAGS) typeinfo\ti_bit.d

ti_byte.obj : typeinfo\ti_byte.d
	$(DC) -c $(DFLAGS) typeinfo\ti_byte.d

ti_ubyte.obj : typeinfo\ti_ubyte.d
	$(DC) -c $(DFLAGS) typeinfo\ti_ubyte.d

ti_short.obj : typeinfo\ti_short.d
	$(DC) -c $(DFLAGS) typeinfo\ti_short.d

ti_ushort.obj : typeinfo\ti_ushort.d
	$(DC) -c $(DFLAGS) typeinfo\ti_ushort.d

ti_int.obj : typeinfo\ti_int.d
	$(DC) -c $(DFLAGS) typeinfo\ti_int.d

ti_uint.obj : typeinfo\ti_uint.d
	$(DC) -c $(DFLAGS) typeinfo\ti_uint.d

ti_long.obj : typeinfo\ti_long.d
	$(DC) -c $(DFLAGS) typeinfo\ti_long.d

ti_ulong.obj : typeinfo\ti_ulong.d
	$(DC) -c $(DFLAGS) typeinfo\ti_ulong.d

ti_float.obj : typeinfo\ti_float.d
	$(DC) -c $(DFLAGS) typeinfo\ti_float.d

ti_cfloat.obj : typeinfo\ti_cfloat.d
	$(DC) -c $(DFLAGS) typeinfo\ti_cfloat.d

ti_ifloat.obj : typeinfo\ti_ifloat.d
	$(DC) -c $(DFLAGS) typeinfo\ti_ifloat.d

ti_double.obj : typeinfo\ti_double.d
	$(DC) -c $(DFLAGS) typeinfo\ti_double.d

ti_cdouble.obj : typeinfo\ti_cdouble.d
	$(DC) -c $(DFLAGS) typeinfo\ti_cdouble.d

ti_idouble.obj : typeinfo\ti_idouble.d
	$(DC) -c $(DFLAGS) typeinfo\ti_idouble.d

ti_real.obj : typeinfo\ti_real.d
	$(DC) -c $(DFLAGS) typeinfo\ti_real.d

ti_creal.obj : typeinfo\ti_creal.d
	$(DC) -c $(DFLAGS) typeinfo\ti_creal.d

ti_ireal.obj : typeinfo\ti_ireal.d
	$(DC) -c $(DFLAGS) typeinfo\ti_ireal.d

ti_Array.obj : typeinfo\ti_Array.d
	$(DC) -c $(DFLAGS) typeinfo\ti_Array.d

ti_AssociativeArray.obj : typeinfo\ti_AssociativeArray.d
	$(DC) -c $(DFLAGS) typeinfo\ti_AssociativeArray.d

ti_StaticArray.obj : typeinfo\ti_StaticArray.d
	$(DC) -c $(DFLAGS) typeinfo\ti_StaticArray.d

ti_AC.obj : typeinfo\ti_AC.d
	$(DC) -c $(DFLAGS) typeinfo\ti_AC.d

ti_Aa.obj : typeinfo\ti_Aa.d
	$(DC) -c $(DFLAGS) typeinfo\ti_Aa.d

ti_Awchar.obj : typeinfo\ti_Awchar.d
	$(DC) -c $(DFLAGS) typeinfo\ti_Awchar.d

ti_Adchar.obj : typeinfo\ti_Adchar.d
	$(DC) -c $(DFLAGS) typeinfo\ti_Adchar.d

ti_Abit.obj : typeinfo\ti_Abit.d
	$(DC) -c $(DFLAGS) typeinfo\ti_Abit.d

ti_Ag.obj : typeinfo\ti_Ag.d
	$(DC) -c $(DFLAGS) typeinfo\ti_Ag.d

ti_Aubyte.obj : typeinfo\ti_Aubyte.d
	$(DC) -c $(DFLAGS) typeinfo\ti_Aubyte.d

ti_Ashort.obj : typeinfo\ti_Ashort.d
	$(DC) -c $(DFLAGS) typeinfo\ti_Ashort.d

ti_Aushort.obj : typeinfo\ti_Aushort.d
	$(DC) -c $(DFLAGS) typeinfo\ti_Aushort.d

ti_Aint.obj : typeinfo\ti_Aint.d
	$(DC) -c $(DFLAGS) typeinfo\ti_Aint.d

ti_Auint.obj : typeinfo\ti_Auint.d
	$(DC) -c $(DFLAGS) typeinfo\ti_Auint.d

ti_Along.obj : typeinfo\ti_Along.d
	$(DC) -c $(DFLAGS) typeinfo\ti_Along.d

ti_Aulong.obj : typeinfo\ti_Aulong.d
	$(DC) -c $(DFLAGS) typeinfo\ti_Aulong.d

ti_Afloat.obj : typeinfo\ti_Afloat.d
	$(DC) -c $(DFLAGS) typeinfo\ti_Afloat.d

ti_Acfloat.obj : typeinfo\ti_Acfloat.d
	$(DC) -c $(DFLAGS) typeinfo\ti_Acfloat.d

ti_Adouble.obj : typeinfo\ti_Adouble.d
	$(DC) -c $(DFLAGS) typeinfo\ti_Adouble.d

ti_Acdouble.obj : typeinfo\ti_Acdouble.d
	$(DC) -c $(DFLAGS) typeinfo\ti_Acdouble.d

ti_Areal.obj : typeinfo\ti_Areal.d
	$(DC) -c $(DFLAGS) typeinfo\ti_Areal.d

ti_Acreal.obj : typeinfo\ti_Acreal.d
	$(DC) -c $(DFLAGS) typeinfo\ti_Acreal.d

ti_Pointer.obj : typeinfo\ti_Pointer.d
	$(DC) -c $(DFLAGS) typeinfo\ti_Pointer.d

ti_Delegate.obj : typeinfo\ti_Delegate.d
	$(DC) -c $(DFLAGS) typeinfo\ti_Delegate.d

ti_Function.obj : typeinfo\ti_Function.d
	$(DC) -c $(DFLAGS) typeinfo\ti_Function.d

ti_Enum.obj : typeinfo\ti_Enum.d
	$(DC) -c $(DFLAGS) typeinfo\ti_Enum.d

ti_Class.obj : typeinfo\ti_Class.d
	$(DC) -c $(DFLAGS) typeinfo\ti_Class.d

ti_Struct.obj : typeinfo\ti_Struct.d
	$(DC) -c $(DFLAGS) typeinfo\ti_Struct.d

ti_Typedef.obj : typeinfo\ti_Typedef.d
	$(DC) -c $(DFLAGS) typeinfo\ti_Typedef.d


######################################################

clean :
	del $(OBJ) $(OBJ_TI)
	del dmdrt.lib

install :
	$(CP) dmdrt.lib ..\..\lib\.
