module std.memory;

private
{
    extern (C) void gc_init();
    extern (C) void gc_term();

    extern (C) void gc_enable();
    extern (C) void gc_disable();
    extern (C) void gc_collect();

    extern (C) void* gc_malloc( size_t sz );
    extern (C) void* gc_calloc( size_t sz );
    extern (C) void gc_free( void* p );

    extern (C) size_t gc_capacityOf( void* p );

    extern (C) void gc_add( void* p );
    extern (C) void gc_add( void* pbeg, void* pend );

    extern (C) void gc_remove( void* p );
//    extern (C) void gc_remove( void* pbeg, void* pend );

    extern (C) void gc_pin( void* p );
    extern (C) void gc_unpin( void* p );
}

struct GC
{
    static void enable()  { gc_enable();  }
    static void disable() { gc_disable(); }
    static void collect() { gc_collect(); }

    static void* malloc( size_t sz ) { return gc_malloc( sz ); }
    static void* calloc( size_t sz ) { return gc_calloc( sz ); }
    static void free( void* p )      { gc_free( p );           }

    static size_t capacityOf( void* p ) { return gc_capacityOf( p ); }

    static void add( void* p )                { gc_add( p );          }
    static void add( void* pbeg, void* pend ) { gc_add( pbeg, pend ); }

    static void remove( void* p )                { gc_remove( p );          }
//    static void remove( void* pbeg, void* pend ) { gc_remove( pbeg, pend ); }

    static void pin( void* p )   { gc_pin( p );   }
    static void unpin( void* p ) { gc_unpin( p ); }
}